@@ -10,7 +10,7 @@ options(width=80)
 args <- commandArgs(trailingOnly = TRUE)
 input = args[1]
 
-if( length(input) == 1 ){
+if( is.na(input) ){
 	cat(paste("Usage: tradis_essentiality.R data.tab\n\n"))
 	cat(strwrap("Produces calls of gene essentiality using an adaptation of the method described in Langridge et al. Genome Research 2009 and Barquist et al. NAR 2013. A loess curve is fit to the distribution of insertion indices, and used to identify the minima between the 'essential' and 'non-essential' distributions. These distributions are then used to fit gamma distributions, which are then used to calculate log-odds ratios, which are used to determine an insertion-index threshold for gene essentiality. Note that this analysis requires a saturated mutant library, and is not suitable for the analysis of data sets with low insertion density. The script produces a number of diagnostic plots which can be used to verify that this condition has been met.\n"), fill=80)
 	q(status=1)
@@ -37,14 +37,14 @@ lo <- loess(h1$density ~ c(1:length(h1$density))) #loess smothing over density
 plot(h1$density, main="Density")
 lines(predict(lo),col='red',lwd=2)
 m = h1$mids[which.min(predict(lo))]
-I1 = ((ii < m)&(ii > 0))
+I1 = ((ii < m)&(ii >= 0))
 
 h = hist(ii, breaks=100,plot=FALSE) 
 I2 = ((ii >= m)&(ii < h$mids[max(which(h$counts>5))]))
 f1 = (sum(I1) + sum(ii == 0))/nG
 f2 = (sum(I2))/nG
 
-d1 = fitdistr(ii[I1], "gamma")
+d1 = fitdistr(ii[I1], "exponential")
 d2 = fitdistr(ii[I2], "gamma") #fit curves
 
 # print pdf of histogram
@@ -52,13 +52,13 @@ d2 = fitdistr(ii[I2], "gamma") #fit curves
 
 #plots
 hist(ii,breaks=200, xlim=c(0,0.1), freq=FALSE,xlab="Insertion index", main="Gamma fits")
-lines(0:50/500, f1*dgamma(0:50/500, 1, d1$estimate[2])) # was [2]
+lines(0:50/500, f1*dgamma(0:50/500, 1, d1$estimate[1])) # was [2]
 lines(0:50/500, f2*dgamma(0:50/500, d2$estimate[1], d2$estimate[2]))
 # print changepoint
 
 #calculate log-odds ratios to choose thresholds
-lower <- max(which(log((pgamma(1:300/10000, d2$e[1],d2$e[2])*(1-pgamma(1:300/10000, 1,d1$e[2], lower.tail=FALSE)))/(pgamma(1:300/10000, 1,d1$e[2], lower.tail=FALSE)*(1-pgamma(1:300/10000, d2$e[1],d2$e[2]))) , base=2) < -2))
-upper <- min(which(log((pgamma(1:300/10000, d2$e[1],d2$e[2])*(1-pgamma(1:300/10000, 1,d1$e[2], lower.tail=FALSE)))/(pgamma(1:300/10000, 1,d1$e[2], lower.tail=FALSE)*(1-pgamma(1:300/10000, d2$e[1],d2$e[2]))) , base=2) > 2))
+lower <- max(which(log((pgamma(1:500/10000, d2$e[1],d2$e[2])*(1-pgamma(1:500/10000, 1,d1$e[1], lower.tail=FALSE)))/(pgamma(1:500/10000, 1,d1$e[1], lower.tail=FALSE)*(1-pgamma(1:500/10000, d2$e[1],d2$e[2]))) , base=2) < -2))
+upper <- min(which(log((pgamma(1:500/10000, d2$e[1],d2$e[2])*(1-pgamma(1:500/10000, 1,d1$e[1], lower.tail=FALSE)))/(pgamma(1:500/10000, 1,d1$e[1], lower.tail=FALSE)*(1-pgamma(1:500/10000, d2$e[1],d2$e[2]))) , base=2) > 2))
 
 essen <- lower/10000
 ambig <- upper/10000
@@ -87,7 +87,7 @@ tradis_essentiality.R - tradis_essentiality.R
 
 =head1 VERSION
 
-version 1.2.2
+version 1.2.4
 
 =head1 AUTHOR
 
