@@ -1,10 +1,11 @@
 package Bio::Tradis::Exception;
 # ABSTRACT: Exceptions for input data 
-$Bio::Tradis::Exception::VERSION = '1.2.2';
+$Bio::Tradis::Exception::VERSION = '1.2.4';
 
 
 use Exception::Class (
-    Bio::Tradis::Exception::RefNotFound => { description => 'Cannot find the reference file' }
+    Bio::Tradis::Exception::RefNotFound    => { description => 'Cannot find the reference file' },
+    Bio::Tradis::Exception::TagFilterError => { description => 'Problem filtering the Fastq by tag' }
 );  
 
 1;
@@ -21,7 +22,7 @@ Bio::Tradis::Exception - Exceptions for input data
 
 =head1 VERSION
 
-version 1.2.2
+version 1.2.4
 
 =head1 SYNOPSIS
 
